"""bioimageio.spec extensions for BioImage.IO community partners"""
# todo: consider moving to its own module, e.g. bioimageio.partner
# todo: if not moving to its own module, add the dependency to a 'partner' extra in setup.py
